from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

import time
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://my.portsmouth.gov.uk/en/AchieveForms/?form_uri=sandbox-publish://AF-Process-26e27e70-f771-47b1-a34d-af276075cede/AF-Stage-cd7cc291-2e59-42cc-8c3f-1f93e132a2c9/definition.json&redirectlink=%2F&cancelRedirectLink=%2F"

        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        check_uprn(user_uprn)
        check_postcode(user_postcode)
        # Create Selenium webdriver
        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property

        iframe_presense = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.ID, "fillform-frame-1"))
        )


        driver.switch_to.frame(iframe_presense)
        wait = WebDriverWait(driver, 60)
        inputElement_postcodesearch = wait.until(
            EC.element_to_be_clickable((By.NAME, "postcode_search"))
        )

        inputElement_postcodesearch.send_keys(user_postcode)
        lookupAddress_btn = wait.until(EC.element_to_be_clickable((By.ID, "lookupAddress")))

        lookupAddress_btn.click()

        

        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = wait.until(EC.element_to_be_clickable((By.NAME, "Choose_Address")))

        dropdown_options = wait.until(
            EC.presence_of_element_located((By.CLASS_NAME, "lookup-option"))
        )
        time.sleep(1)
        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value(str(user_uprn))


        h4_element = wait.until(
            EC.presence_of_element_located((By.XPATH, "//h4[contains(text(), 'next 10 collection dates')]"))
        )
        
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        # Define your XPath
        elements_with_data_field_name = soup.find_all(lambda tag: tag.has_attr('data-field-name') and tag['data-field-name'].startswith('html'))
        if elements_with_data_field_name:

            for element in elements_with_data_field_name:
                # Extract h4 text from the current element
                h4_text = element.find('h4').get_text(strip=True) if element.find('h4') else None
                
                # Process the data (dates) in the current element (p tags)
                    
                if h4_text:
                    if "next 10" in h4_text:
                        data_paragraphs = element.find_all('p') if element else []

                        # Extract dates from the first <p> tag (assuming dates are in the first <p> tag)
                        dates_paragraph = data_paragraphs[0] if len(data_paragraphs) > 0 else None
                        dates = dates_paragraph.find_all(string=True, recursive=False) if dates_paragraph else []
                        
                        for date in dates:
                            data["bins"].append(
                                {
                                    "type": h4_text.split(' - ')[0],
                                    "collectionDate": datetime.strptime(re.sub(r'[^a-zA-Z0-9,\s]', '', date).strip(), "%A %d %B %Y").strftime("%d/%m/%Y")
                                }
                            )

        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://community.fdean.gov.uk/s/waste-collection-enquiry"

        data = {"bins": []}

        house_number = kwargs.get("paon")
        postcode = kwargs.get("postcode")
        full_address = f"{house_number}, {postcode}" 
        web_driver = kwargs.get("web_driver")

        # Create Selenium webdriver
        driver = create_webdriver(web_driver)
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property
        wait = WebDriverWait(driver, 60)
        address_entry_field = wait.until(
            EC.presence_of_element_located((By.XPATH, '//*[@id="combobox-input-19"]'))
        )

        address_entry_field.send_keys(str(full_address))

        address_entry_field = wait.until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="combobox-input-19"]'))
        )
        address_entry_field.click()
        address_entry_field.send_keys(Keys.BACKSPACE)
        address_entry_field.send_keys(str(full_address[len(full_address)-1]))

        first_found_address = wait.until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="dropdown-element-19"]/ul'))
        )

        first_found_address.click()
        # Wait for the 'Select your property' dropdown to appear and select the first result
        next_btn = wait.until(EC.element_to_be_clickable((By.XPATH, "//lightning-button/button")))
        next_btn.click()
        bin_data = wait.until(
            EC.presence_of_element_located((By.XPATH, "//span[contains(text(), 'Container')]"))
        )

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        rows = soup.find_all('tr', class_='slds-hint-parent')
        current_year = datetime.now().year

        for row in rows:
            columns = row.find_all('td')
            if columns:
                container_type = row.find('th').text.strip()
                collection_day = re.sub(r'[^a-zA-Z0-9,\s]', '', columns[0].get_text()).strip()

                # Parse the date from the string
                parsed_date = datetime.strptime(collection_day, '%a, %d %B')
                if parsed_date < datetime(parsed_date.year, parsed_date.month, parsed_date.day):
                    parsed_date = parsed_date.replace(year=current_year + 1)
                else:
                    parsed_date = parsed_date.replace(year=current_year)
                # Format the date as %d/%m/%Y
                formatted_date = parsed_date.strftime('%d/%m/%Y')
                
                # Add the bin type and collection date to the 'data' dictionary
                data["bins"].append(
                    {
                        "type": container_type,
                        "collectionDate": formatted_date
                    }
                )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from lxml import etree


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="lxml")
        soup.prettify()
        data = {"bins": []}
        collections = []

        # Convert the XML to JSON and load the next collection data
        result = soup.find("p").contents[0].text.replace("\\", "")[1:-1]
        json_data = json.loads(result)['NextCollection']

        # Get general waste data
        if json_data.get('RefuseCollectionBinDate') is not None:
            bin_type = 'Green general waste bin'
            if json_data.get('RefuseBinExceptionMessage') != "":
                bin_type += f" ({json_data.get('RefuseBinExceptionMessage')})".rstrip()
            bin_date = datetime.strptime(json_data.get('RefuseCollectionBinDate'), "%Y-%m-%dT%H:%M:%S")
            collections.append((bin_type, bin_date))

        # Get recycling waste data
        if json_data.get('RecyclingCollectionDate') is not None:
            bin_type = 'Blue recycling bin'
            if json_data.get('RecyclingExceptionMessage') != "":
                bin_type += f" ({json_data.get('RecyclingExceptionMessage')})".rstrip()
            bin_date = datetime.strptime(json_data.get('RecyclingCollectionDate'), "%Y-%m-%dT%H:%M:%S")
            collections.append((bin_type, bin_date))

        # Get garden waste data
        if json_data.get('GardenWasteCollectionDate') is not None:
            bin_type = 'Brown garden waste bin'
            if json_data.get('GardenWasteExceptionMessage') != "":
                bin_type += f" ({json_data.get('GardenWasteExceptionMessage')})".rstrip()
            bin_date = datetime.strptime(json_data.get('GardenWasteCollectionDate'), "%Y-%m-%dT%H:%M:%S")
            collections.append((bin_type, bin_date))

        # Get food waste data
        if json_data.get('FoodWasteCollectionDate') is not None:
            bin_type = 'Black food waste bin'
            if json_data.get('FoodWasteExceptionMessage') != "":
                bin_type += f" ({json_data.get('FoodWasteExceptionMessage')})".rstrip()
            bin_date = datetime.strptime(json_data.get('FoodWasteCollectionDate'), "%Y-%m-%dT%H:%M:%S")
            collections.append((bin_type, bin_date))

        # If there's no collections, raise an error
        if len(collections) < 1:
            raise ValueError("No collections found")

        # Order the collection by date, then return them
        ordered_data = sorted(collections, key=lambda x: x[1])
        for bin in ordered_data:
            dict_data = {
                "type": bin[0],
                "collectionDate": bin[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data
